package com.xilinx.sdk.terminal;

import jssc.SerialPortList;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

/**
 * @author ravimish
 *
 */
public class SerialConnectionDialog extends Dialog
{

    private ComboViewer SerialPorts;
    private ComboViewer BaudRate;
    private String SelectedPort;
    private int SelectedRate;
	private ComboViewer dataBit;

	private ComboViewer stopBits;
	

	private ComboViewer parity;
	private ComboViewer flowCtl;
	private Text timeTxt;
	private int selectedDataBit;
	private float selectedStopBit;
	private char selectedParity;

    protected SerialConnectionDialog(Shell parentShell) {
    	super(parentShell);
    	//setShellStyle(getShellStyle() | SWT.RESIZE);

    }

    @Override
	    protected Control createDialogArea(final Composite parent) {
			GridLayout layout = new GridLayout();
			layout.numColumns = 2;
			layout.marginLeft = 8;
			layout.marginRight = 8;			
			// layout.horizontalAlignment = GridData.FILL;
			parent.setLayout(layout);			
			parent.setLayoutData(new GridData(SWT.FILL,SWT.NONE,true,true));
		
			Group basicGroup = new Group(parent, SWT.NONE);
			basicGroup.setLayout(new GridLayout(2, false));
			GridData gridData = new GridData(SWT.FILL, SWT.NONE, true, true);
			gridData.horizontalSpan = 2;
			basicGroup.setLayoutData(gridData);
			basicGroup.setText("Basic Settings");
			
			
			// Create the serial port combo box to allow to select a serial port
			Label label1 = new Label(basicGroup, SWT.NONE);
			label1.setText("Port:");				
			
			SerialPorts = new ComboViewer(basicGroup, SWT.DROP_DOWN);
			gridData = new GridData(SWT.FILL, SWT.NONE, true, false);
			gridData.widthHint = 75;
			SerialPorts.getControl().setLayoutData(gridData);
			SerialPorts.setContentProvider(new ArrayContentProvider());
			SerialPorts.setLabelProvider(new LabelProvider());
			SerialPorts.setInput(getAllPorts());
			SerialPorts.getCombo().setText(TerminalUtils.getGlobalString(TerminalUtils.SERIALPORT_KEY, ""));
	
			// Create baud rate selection combo box to select the baud rate
			Label label2 = new Label(basicGroup, SWT.NONE);
			label2.setText("Baud Rate:");
			
			BaudRate = new ComboViewer(basicGroup, SWT.READ_ONLY | SWT.DROP_DOWN);
			BaudRate.getControl().setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
			BaudRate.setContentProvider(new ArrayContentProvider());
			BaudRate.setLabelProvider(new LabelProvider());
			BaudRate.setInput(new String[]{  "115200", "57600", "38400", "31250", "28800", "19200", "14400", "9600", "4800", "2400", "1200", "300" });
			BaudRate.getCombo().setText("115200");
		
			// create advance settings section
			
			 FormToolkit toolkit = new FormToolkit(parent.getDisplay());			
			 Section advanceSection = toolkit.createSection(parent,Section.DESCRIPTION|Section.TITLE_BAR|
					  Section.TWISTIE );
			 advanceSection.setText("Advance Settings");
			 advanceSection.setEnabled(true);
			 advanceSection.setLayout(layout);
			 advanceSection.setExpanded(true);
			 gridData = new GridData(SWT.FILL, SWT.NONE, true, true);
			 gridData.horizontalSpan = 2;
			 advanceSection.setLayoutData(gridData);
			 
			 
			 
			 Composite client = toolkit.createComposite(advanceSection, SWT.WRAP);
			 layout = new GridLayout();
			 layout.numColumns = 2;
			 client.setLayout(layout);
			 // create data bits
			 
			 Label dataBits  = new Label(client, SWT.NONE);
			 dataBits.setText("Data Bits:");
			 
			 dataBit  = new ComboViewer(client, SWT.READ_ONLY|SWT.DROP_DOWN);
			 dataBit.getControl().setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, true));
			 dataBit.setContentProvider(new ArrayContentProvider());
			 dataBit.setInput(new String[]{"5","6","7","8"});
			 dataBit.getCombo().setText("8");
			 
			 // stop bits
			 Label stopBitsLabel  = new Label(client, SWT.NONE);
			 stopBitsLabel.setText("Stop Bits:");
			 
			 stopBits  = new ComboViewer(client, SWT.READ_ONLY|SWT.DROP_DOWN);
			 stopBits.getControl().setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, true));
			 stopBits.setContentProvider(new ArrayContentProvider());
			 stopBits.setInput(new String[]{"1","1.5","2"});
			 stopBits.getCombo().setText("1");
			 
			 // parity
			 Label parityLabel  = new Label(client, SWT.NONE);
			 parityLabel.setText("Parity:");
			 
			 parity  = new ComboViewer(client, SWT.READ_ONLY|SWT.DROP_DOWN);
			 parity.getControl().setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, true));
			 parity.setContentProvider(new ArrayContentProvider());
			 parity.setInput(new String[]{"None","Even","Odd","Mark","Space"});
			 parity.getCombo().setText("None");
			 
			 // Flow control
	
			 Label flowCtlLabel  = new Label(client, SWT.NONE);
			 flowCtlLabel.setText("Flow Control:");
			 
			 flowCtl  = new ComboViewer(client, SWT.READ_ONLY|SWT.DROP_DOWN);
			 flowCtl.getControl().setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, true));
			 flowCtl.setContentProvider(new ArrayContentProvider());
			 flowCtl.setInput(new String[]{"None","RTS/CTS","Xon/Xoff"});
			 flowCtl.getCombo().setText("None");
			 
			 // Time out 
			 Label timeOut  = new Label(client, SWT.NONE);
			 timeOut.setText("Timeout (sec):");
			 
			 timeTxt = new Text(client, SWT.BORDER);
			 timeTxt.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, true));
			 
			 
			 
			 advanceSection.setClient(client);
			 advanceSection.addExpansionListener(new ExpansionAdapter() {
				  public void expansionStateChanged(ExpansionEvent e) {
					  parent.redraw();
				  }
				 });
			 
			 
			return parent;
	
	    }
    
    private String[] getAllPorts() {
    	return SerialPortList.getPortNames();
	}

	protected void configureShell(Shell shell) {
        super.configureShell(shell);
        shell.setText("Connect to serial port");
     }
    
    
    
    @Override
    protected void okPressed() {
    	SelectedRate = Integer.parseInt(BaudRate.getCombo().getText());
    	SelectedPort = SerialPorts.getCombo().getText();    	
    	TerminalUtils.setGlobalValue(TerminalUtils.SERIALPORT_KEY, SerialPorts.getCombo().getText());
    	selectedDataBit = Integer.parseInt(dataBit.getCombo().getText());
    	selectedStopBit = Float.parseFloat(stopBits.getCombo().getText());
    	selectedParity = parity.getCombo().getText().charAt(0);
    	super.okPressed();
    }
    
	public int getDataBit() {		
		return selectedDataBit;
	}

	public float getStopBits() {
		return selectedStopBit;
	}

	public char getParity() {
		return selectedParity;
	}

    

    public String getSerialPort() {
    	return SelectedPort;
    }

    public int getBaudRate() {
    	return SelectedRate;
    }

}

// XSIP watermark, do not delete 67d7842dbbe25473c3c32b93c0da8047785f30d78e8a024de1b57352245f9689
