package com.xilinx.sdk.terminal;

import java.util.ArrayList;
import java.util.List;

import jssc.SerialPort;
import jssc.SerialPortEvent;
import jssc.SerialPortEventListener;
import jssc.SerialPortException;

/**
 * @author ravimish
 * 
 */
public class SerialPortWrapper implements SerialPortEventListener {

	SerialPort serialPort;
	StringBuilder messages = new StringBuilder();
	List<IListener> consumers = new ArrayList<IListener>();
	public String connectionMsg = "";

	int rate;
	int parity;
	int databits;
	int stopbits;
	String portName;

	public String getPortName() {
		return portName;
	}

	public SerialPortWrapper() {

	}

	public int getRate() {
		return rate;
	}

	public int getParity() {
		return parity;
	}

	public int getDatabits() {
		return databits;
	}

	public SerialPortWrapper(String name, int rate) throws SerialPortException {
		this(name, rate, 1.0f,8, 'N');
	}

	public SerialPortWrapper(String name, int rate, float stopbits , int dataBits, char parity
			) throws SerialPortException {
		portName = name;
		this.rate = rate;


		parity = SerialPort.PARITY_NONE;
		if (parity == 'E')
		    parity = SerialPort.PARITY_EVEN;
		if (parity == 'O')
		    parity = SerialPort.PARITY_ODD;


		stopbits = SerialPort.STOPBITS_1;
		if (stopbits == 1.5f)
		    stopbits = SerialPort.STOPBITS_1_5;
		if (stopbits == 2)
		    stopbits = SerialPort.STOPBITS_2;

		this.databits = dataBits;

		connect();

	}

	@Override
	public void serialEvent(SerialPortEvent event) {
		if (event.getEventType() == SerialPortEvent.RXCHAR) {
			int eventValue = event.getEventValue();
			try {
				byte[] bytes = serialPort.readBytes(eventValue);
				for (byte b : bytes) {
					messages.append((char) b);
				}
				notifyConsumersOfEvent(new String(bytes));
			} catch (SerialPortException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
	}

	public void addListener(IListener consumer) {
		consumers.add(consumer);
	}

	public void removeListener(IListener consumer) {
		if (consumer != null)
			consumers.add(consumer);
	}

	public void notifyView(String data) {
		for (IListener consumer : consumers) {
			consumer.event(data);
		}

	}

	private void notifyConsumersOfEvent(String message) {
		if (consumers != null) {
			for (IListener consumer : consumers) {
				consumer.event(message);
			}
		}
	}

	public void write(String str, String LineEnd) {
		String updatedStr = str;// "\r\n"+str+"\r\n";

		// write(updatedStr.getBytes(Charset.forName("ISO-8859-1")));
		write(updatedStr.getBytes());

	}

	public void write(byte bytes[]) {
		if (serialPort != null) {
			try {
				serialPort.writeBytes(bytes);
				char r_char = '\r';
				serialPort.writeByte((byte) r_char);
				// serialPort.writeByte((byte)'\n');
				serialPort.purgePort(SerialPort.MASK_RXCHAR
						| SerialPort.MASK_BREAK);
				// serialPort.writeByte((byte)0x0D);
				// serialPort.writeByte((byte)0x0A);
			} catch (SerialPortException e) {
				e.printStackTrace();
			}
		}
	}

	public void dispose() {
		notifyConsumersOfEvent(System.getProperty("line.separator")+"Disconnect of port " + serialPort.getPortName()
				+ " executed"+System.getProperty("line.separator"));
		disconnect();

	}

	public void disconnect() {
		if (serialPort != null) {
			try {
				serialPort.closePort();
			} catch (SerialPortException e) {
				e.printStackTrace();
			}
			serialPort = null;
		}
	}

	public boolean isConnected() {
		return (serialPort != null && serialPort.isOpened());
	}

	public void connect() {
		if (serialPort == null) {

			serialPort = new SerialPort(portName);
			
			try {
				serialPort.openPort();
				serialPort.setParams(rate, databits, stopbits, parity);
				serialPort.setEventsMask(SerialPortEvent.CTS);
				int eventMask = SerialPort.MASK_RXCHAR | SerialPort.MASK_BREAK;
				serialPort.addEventListener(this, eventMask);
			} catch (SerialPortException e) {
				String exceptionType = e.getExceptionType();
				if (exceptionType.equalsIgnoreCase("port busy")) {
					connectionMsg = "Already connected to port: " + serialPort.getPortName();
				} else
					connectionMsg = "Nothing found at port: " + serialPort.getPortName();
			}
			
			return;

		}
	}

}

// XSIP watermark, do not delete 67d7842dbbe25473c3c32b93c0da8047785f30d78e8a024de1b57352245f9689
